<?php

namespace Drupal\bibcite_import_orcid;

use Drupal;
use Drupal\user\Entity\User;
use Drupal\bibcite_entity\Entity\Contributor;
use RenanBr\BibTexParser\Listener;
use RenanBr\BibTexParser\Parser;
use RenanBr\BibTexParser\Processor;
use RenanBr\BibTexParser\Exception\ParserException;
use RenanBr\BibTexParser\Exception\ProcessorException;

/**
 * Import ORCID data.
 */
class Import {

  public function getOrcidWorkData($work_put_code, $orcid)
  {
      $url = "https://pub.orcid.org/" . $orcid . "/works/" . $work_put_code;

      $ch = curl_init();

      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
      curl_setopt($ch, CURLOPT_TIMEOUT, 600);
      curl_setopt($ch, CURLOPT_HTTPHEADER, [
          'Accept: application/vnd.orcid+json',
      ]);

      $curl_result = curl_exec($ch);

      if (curl_error($ch)) {
          $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
          Drupal::logger('bibcite_import_orcid')->notice("cURL error while fetching ORCID works for user " . $orcid . ": " . curl_error($ch));
          if ($http_code == 504) {
              Drupal::logger('bibcite_import_orcid')->notice("ORCID API Timeout");
          }
      }

      curl_close($ch);

      $json_data = json_decode($curl_result, true);

      return $json_data['bulk'][0]['work'];
  }
  public function getOrcidPersonData($orcid)
  {
    //Set the url for ORCID user data
    $url = "https://pub.orcid.org/" . $orcid;
    //Initialize cURL session
    $ch = curl_init();
    //Define cURL options
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, 600);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Accept: application/vnd.orcid+json',
    ]);
    //Receive the result of the cURL session
    $curl_result = curl_exec($ch);
    //Check if there were any errors during the curl session
    if (curl_error($ch)) {
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        Drupal::logger('bibcite_import_orcid')->notice("cURL error while fetching ORCID works for user " . $orcid . ": " . curl_error($ch));
        if ($http_code == 504) {
            Drupal::logger('bibcite_import_orcid')->notice("ORCID API Timeout");
        }
    }
    //Close cURL session
    curl_close($ch);

    //Decodes the information to JSON
    $json_data = json_decode($curl_result, true);
    //Receive the data from the user
    $person_data = $json_data['person'];

    return $person_data;
  }
  // Map ref types
  public function bibcite_entity_types($entity){
    $pubs = [
        "annotation" => "miscellaneous",
        "artistic-performance" => "miscellaneous",
        "book-chapter" => "book_chapter",
        "book-review" => "miscellaneous",
        "book" => "book",
        "conference-abstract" => "conference_proceedings",
        "conference-paper" => "conference_paper",
        "conference-poster" => "miscellaneous",
        "data-set" => "database",
        "dictionary-entry" => "miscellaneous",
        "disclosure" => "miscellaneous",
        "dissertation-thesis" => "thesis",
        "edited-book" => "book",
        "encyclopedia-entry" => "miscellaneous",
        "invention" => "miscellaneous",
        "journal-article" => "journal_article",
        "journal-issue" => "journal",
        "lecture-speech" => "presentation",
        "license" => "miscellaneous",
        "magazine-article" => "magazine_article",
        "manual" => "miscellaneous",
        "newsletter-article" => "miscellaneous",
        "newspaper-article" => "newspaper_article",
        "online-resource" => "website",
        "other" => "miscellaneous",
        "patent" => "patent",
        "physical-object" => "miscellaneous",
        "presentation" => "presentation",
        "preprint" => "preprint",
        "registered-copyright" => "miscellaneous",
        "report" => "report",
        "research-technique" => "miscellaneous",
        "research-tool" => "miscellaneous",
        "software" => "software",
        "spin-off-company" => "miscellaneous",
        "standards-and-policy" => "miscellaneous",
        "supervised-student-publication" => "miscellaneous",
        "technical-standard" => "miscellaneous",
        "test" => "miscellaneous",
        "trademark" => "miscellaneous",
        "translation" => "miscellaneous",
        "website" => "website",
        "working-paper" => "miscellaneous",
        "undefined" => "miscellaneous",
    ];
    if (array_key_exists($entity, $pubs)) {
      return $pubs[$entity];
    } else {
        return "miscellaneous";
    }
  }
  public function parce_bibtex($work){
    // handle bibtex if it exists
    try {
      $parser = new Parser();
      $listener = new Listener();
      $listener->addProcessor(new Processor\TagNameCaseProcessor(CASE_LOWER));
      $listener->addProcessor(new Processor\NamesProcessor());

      $parser->addListener($listener);
      $parser->parseString($work);
      $reference_data = [];
      $entries = $listener->export();

      foreach ($entries[0] as $prop => $value) {
          switch ($prop) {
              case 'pages':
              case 'Pages':
              case 'PAGES':
                $reference_data['bibcite_pages'] = $entries[0][$prop];
                break;
              case 'number':
              case 'Number':
              case 'NUMBER':
                $reference_data['bibcite_issue'] = $entries[0][$prop];
                break;
              case 'volume':
              case 'Volume':
              case 'VOLUME':
                $reference_data['bibcite_volume'] = $entries[0][$prop];
                break;
              case 'author':
              case 'Author':
              case 'AUTHOR':
                $reference_data['bibcite_authors'] = $entries[0][$prop];
                break;
              case 'booktitle':
              case 'Booktitle':
              case 'BookTitle':
              case 'BOOKTITLE':
                $reference_data['bibcite_booktitle'] = $entries[0][$prop];
                break;
              case 'series':
              case 'Series':
              case 'SERIES':
                $reference_data['bibcite_section'] = $entries[0][$prop];
                break;
              case 'publisher':
              case 'Publisher':
              case 'PUBLISHER':
                $reference_data['bibcite_publisher'] = $entries[0][$prop];
                break;
          }
      }
      return $reference_data;
    } catch (ParserException $exception) {
        Drupal::logger('bibcite_import_orcid')->error("Parser Exception: " . $exception->getMessage());
    } catch (ProcessorException $exception) {
        Drupal::logger('bibcite_import_orcid')->error("Processor Exception: " . $exception->getMessage());
    }
  }
  // Get a name and return id (existing or created)
  public function author_to_id($name){
    if(!is_int($name)){
      // Parse the name
      if(str_contains($name,',')){
        $names = explode(',', $name);
        $cfname = ucfirst(end($names));
        $clname = reset($names);
      } else {
        $names = explode(" ", $name);
        $cfname = reset($names);
        $clname = ucfirst(end($names));
      }
      $label = $cfname . ' ' . $clname;

      // Check if the contributor exists
      $query = Drupal::entityQuery('bibcite_contributor')
      ->condition('first_name', $cfname)
      ->condition('last_name', $clname)
      ->execute();

      if(!empty($query)){
        // Check if there's already an attached author with the same name
        $user_id = \Drupal::currentUser()->id();
        $user = User::load($user_id);
        $author_ids = $user->field_author->getValue();
        $author_labels = [];
        foreach ($author_ids as $author_id) {
          $auth_id = $author_id["target_id"];
          $storage = \Drupal::entityTypeManager()->getStorage("bibcite_contributor");
          $author = $storage->load($auth_id);
          if($author){
            $author_labels[] = $author->label();
          }
        }
        if (in_array(trim($label),$author_labels)) {
          return reset($author_id);
        }
      }
      //Create a contributor
      $c = Contributor::create([
        'type' => 'entity_contributor',
        'last_name' => $clname,
        'first_name' => $cfname,
      ]);
      $c->save();
      $id = $c->id();
      return $id;
    }
  }
}
